@@ -10,6 +10,7 @@ import (
 	"path"
 	"strings"
 	"sync"
+	"sync/atomic"
 	"time"
 
 	"github.com/alecthomas/chroma/v2"
@@ -23,18 +24,24 @@ import (
 //
 // This package provides query methods for the struct, no peeking!!
 type Reader struct {
+	sync.Mutex
+
 	lines   []*Line
 	name    *string
-	lock    *sync.Mutex
 	err     error
 	_stderr io.Reader
 
 	// Have we had our contents replaced using setText()?
 	replaced bool
 
-	done             chan bool
-	highlightingDone chan bool // Used by tests
-	moreLinesAdded   chan bool
+	done             *atomic.Bool
+	highlightingDone *atomic.Bool
+
+	// For telling the UI it should recheck the --quit-if-one-screen conditions.
+	// Signalled when either highlighting is done or reading is done.
+	maybeDone chan bool
+
+	moreLinesAdded chan bool
 }
 
 // InputLines contains a number of lines from the reader, plus metadata
@@ -50,15 +57,23 @@ type InputLines struct {
 
 // Shut down the filter (if any) after we're done reading the file.
 func (reader *Reader) cleanupFilter(fromFilter *exec.Cmd) {
+	defer func() {
+		reader.done.Store(true)
+		select {
+		case reader.maybeDone <- true:
+		default:
+		}
+	}()
+
 	// FIXME: Close the stream now that we're done reading it?
 
 	if fromFilter == nil {
-		reader.done <- true
+		log.Trace("Reader done, no filter")
 		return
 	}
 
-	reader.lock.Lock()
-	defer reader.lock.Unlock()
+	reader.Lock()
+	defer reader.Unlock()
 
 	// Give the filter a little time to go away
 	timer := time.AfterFunc(2*time.Second, func() {
@@ -93,15 +108,7 @@ func (reader *Reader) cleanupFilter(fromFilter *exec.Cmd) {
 
 	// FIXME: Report any filter printouts to stderr to the user
 
-	// Must send non-blocking since the channel has no buffer and sometimes no reader
-	select {
-	case reader.done <- true:
-	default:
-		// Empty default statement required for the write to be non-blocking,
-		// without this the write blocks and just hangs. Then we never get to
-		// the deferred reader.lock.Unlock() (see above), and the pager hangs
-		// when trying to take the lock for getting more lines.
-	}
+	log.Trace("Reader done, filter done")
 }
 
 // Count lines in the original file and preallocate space for them.  Good
@@ -115,8 +122,8 @@ func (reader *Reader) preAllocLines(originalFileName string) {
 		return
 	}
 
-	reader.lock.Lock()
-	defer reader.lock.Unlock()
+	reader.Lock()
+	defer reader.Unlock()
 
 	if len(reader.lines) == 0 {
 		// We had no lines since before, this is the expected happy path.
@@ -162,12 +169,12 @@ func (reader *Reader) readStream(stream io.Reader, originalFileName *string, fro
 					break
 				}
 
-				reader.lock.Lock()
+				reader.Lock()
 				if reader.err == nil {
 					// Store the error unless it overwrites one we already have
 					reader.err = fmt.Errorf("error reading line from input stream: %w", err)
 				}
-				reader.lock.Unlock()
+				reader.Unlock()
 				break
 			}
 
@@ -185,14 +192,14 @@ func (reader *Reader) readStream(stream io.Reader, originalFileName *string, fro
 		newLineString := string(completeLine)
 		newLine := NewLine(newLineString)
 
-		reader.lock.Lock()
+		reader.Lock()
 		if reader.replaced {
 			// Somebody called setText(), never mind reading the rest of this stream
-			reader.lock.Unlock()
+			reader.Unlock()
 			break
 		}
 		reader.lines = append(reader.lines, &newLine)
-		reader.lock.Unlock()
+		reader.Unlock()
 		completeLine = completeLine[:0]
 
 		// This is how to do a non-blocking write to a channel:
@@ -217,12 +224,12 @@ func (reader *Reader) readStream(stream io.Reader, originalFileName *string, fro
 // corner to help the user keep track of what is being paged.
 func NewReaderFromStream(name string, reader io.Reader) *Reader {
 	mReader := newReaderFromStream(reader, nil, nil)
-	mReader.highlightingDone <- true // No highlighting of streams = nothing left to do = Done!
+	mReader.highlightingDone.Store(true) // No highlighting of streams = nothing left to do = Done!
 
 	if len(name) > 0 {
-		mReader.lock.Lock()
+		mReader.Lock()
 		mReader.name = &name
-		mReader.lock.Unlock()
+		mReader.Unlock()
 	}
 
 	return mReader
@@ -238,14 +245,18 @@ func NewReaderFromStream(name string, reader io.Reader) *Reader {
 // If fromFilter is not nil this method will wait() for it, and effectively
 // takes over ownership for it.
 func newReaderFromStream(reader io.Reader, originalFileName *string, fromFilter *exec.Cmd) *Reader {
+	done := atomic.Bool{}
+	done.Store(false)
+	highlightingDone := atomic.Bool{}
+	highlightingDone.Store(false)
 	returnMe := Reader{
-		lock: new(sync.Mutex),
-		done: make(chan bool, 1),
 		// This needs to be size 1. If it would be 0, and we add more
 		// lines while the pager is processing, the pager would miss
 		// the lines added while it was processing.
 		moreLinesAdded:   make(chan bool, 1),
-		highlightingDone: make(chan bool, 1),
+		maybeDone:        make(chan bool, 1),
+		highlightingDone: &highlightingDone,
+		done:             &done,
 	}
 
 	// FIXME: Make sure that if we panic somewhere inside of this goroutine,
@@ -271,19 +282,17 @@ func NewReaderFromText(name string, text string) *Reader {
 			lines = append(lines, &line)
 		}
 	}
-	done := make(chan bool, 1)
-	highlightingDone := make(chan bool, 1)
-	done <- true
+	done := atomic.Bool{}
+	done.Store(true)
+	highlightingDone := atomic.Bool{}
+	highlightingDone.Store(true) // No highlighting to do = nothing left = Done!
 	returnMe := &Reader{
 		name:             &name,
 		lines:            lines,
-		lock:             &sync.Mutex{},
-		done:             done,
-		highlightingDone: highlightingDone,
+		done:             &done,
+		highlightingDone: &highlightingDone,
 	}
 
-	returnMe.highlightingDone <- true // No highlighting to do = nothing left = Done!
-
 	return returnMe
 }
 
@@ -310,11 +319,11 @@ func newReaderFromCommand(filename string, filterCommand ...string) (*Reader, er
 	}
 
 	reader := newReaderFromStream(filterOut, nil, filter)
-	reader.highlightingDone <- true // No highlighting to do == nothing left == Done!
-	reader.lock.Lock()
+	reader.highlightingDone.Store(true) // No highlighting to do == nothing left == Done!
+	reader.Lock()
 	reader.name = &filename
 	reader._stderr = filterErr
-	reader.lock.Unlock()
+	reader.Unlock()
 	return reader, nil
 }
 
@@ -387,7 +396,7 @@ func countLines(filename string) (uint64, error) {
 	t1 := time.Now().UnixNano()
 	dtNanos := t1 - t0
 	if count == 0 {
-		log.Debug("Counted ", count, " lines in ", dtNanos/1_000_000, "ms")
+		log.Debug("Counted ", count, " lines in 0ms")
 	} else {
 		log.Debug("Counted ", count, " lines in ", dtNanos/1_000_000, "ms at ", dtNanos/int64(count), "ns/line")
 	}
@@ -421,13 +430,19 @@ func NewReaderFromFilename(filename string, style chroma.Style, formatter chroma
 	}
 
 	returnMe := newReaderFromStream(stream, &filename, nil)
-	returnMe.lock.Lock()
+	returnMe.Lock()
 	returnMe.name = &filename
-	returnMe.lock.Unlock()
+	returnMe.Unlock()
 
 	go func() {
 		defer func() {
-			returnMe.highlightingDone <- true
+			returnMe.highlightingDone.Store(true)
+			select {
+			case returnMe.maybeDone <- true:
+			default:
+			}
+
+			log.Trace("Highlighting done")
 		}()
 
 		highlighted, err := highlight(filename, false, style, formatter)
@@ -472,16 +487,16 @@ func (r *Reader) createStatusUnlocked(lastLineOneBased int) string {
 
 // GetLineCount returns the number of lines available for viewing
 func (r *Reader) GetLineCount() int {
-	r.lock.Lock()
-	defer r.lock.Unlock()
+	r.Lock()
+	defer r.Unlock()
 
 	return len(r.lines)
 }
 
 // GetLine gets a line. If the requested line number is out of bounds, nil is returned.
 func (r *Reader) GetLine(lineNumberOneBased int) *Line {
-	r.lock.Lock()
-	defer r.lock.Unlock()
+	r.Lock()
+	defer r.Unlock()
 
 	if lineNumberOneBased < 1 {
 		return nil
@@ -493,26 +508,30 @@ func (r *Reader) GetLine(lineNumberOneBased int) *Line {
 }
 
 // GetLines gets the indicated lines from the input
-func (r *Reader) GetLines(firstLineOneBased int, wantedLineCount int) *InputLines {
-	r.lock.Lock()
-	defer r.lock.Unlock()
+//
+// Overflow state will be didFit if we returned all lines we currently have, or
+// didOverflow otherwise.
+func (r *Reader) GetLines(firstLineOneBased int, wantedLineCount int) (*InputLines, overflowState) {
+	r.Lock()
+	defer r.Unlock()
 	return r.getLinesUnlocked(firstLineOneBased, wantedLineCount)
 }
 
-func (r *Reader) getLinesUnlocked(firstLineOneBased int, wantedLineCount int) *InputLines {
+func (r *Reader) getLinesUnlocked(firstLineOneBased int, wantedLineCount int) (*InputLines, overflowState) {
 	if firstLineOneBased < 1 {
 		firstLineOneBased = 1
 	}
 
 	if len(r.lines) == 0 {
 		return &InputLines{
-			lines: nil,
+				lines: nil,
 
-			// The line number set here won't matter, we'll clip it anyway when we get it back
-			firstLineOneBased: 0,
+				// The line number set here won't matter, we'll clip it anyway when we get it back
+				firstLineOneBased: 0,
 
-			statusText: r.createStatusUnlocked(0),
-		}
+				statusText: r.createStatusUnlocked(0),
+			},
+			didFit // Empty files always fit
 	}
 
 	firstLineZeroBased := firstLineOneBased - 1
@@ -534,11 +553,18 @@ func (r *Reader) getLinesUnlocked(firstLineOneBased int, wantedLineCount int) *I
 		return r.getLinesUnlocked(firstLineOneBased, wantedLineCount)
 	}
 
-	return &InputLines{
-		lines:             r.lines[firstLineZeroBased : lastLineZeroBased+1],
-		firstLineOneBased: firstLineOneBased,
-		statusText:        r.createStatusUnlocked(lastLineZeroBased + 1),
+	returnLines := r.lines[firstLineZeroBased : lastLineZeroBased+1]
+	overflow := didFit
+	if len(returnLines) != len(r.lines) {
+		overflow = didOverflow // We're not returning all available lines
 	}
+
+	return &InputLines{
+			lines:             returnLines,
+			firstLineOneBased: firstLineOneBased,
+			statusText:        r.createStatusUnlocked(lastLineZeroBased + 1),
+		},
+		overflow
 }
 
 // Replace reader contents with the given text and mark as done
@@ -555,15 +581,17 @@ func (reader *Reader) setText(text string) {
 		lines = lines[0 : len(lines)-1]
 	}
 
-	reader.lock.Lock()
+	reader.Lock()
 	reader.lines = lines
 	reader.replaced = true
-	reader.lock.Unlock()
+	reader.Unlock()
 
+	reader.done.Store(true)
 	select {
-	case reader.done <- true:
+	case reader.maybeDone <- true:
 	default:
 	}
+	log.Trace("Reader done, contents explicitly set")
 
 	select {
 	case reader.moreLinesAdded <- true:
